﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;
using System.CodeDom.Compiler;
using System.IO;
using Microsoft.CSharp;
using System.Reflection;



namespace Project {



	/// <summary>
	/// Tests
	/// </summary>
	public static class Tests {



		/// <summary>
		/// Test script
		/// </summary>
		public static string code =
@"using System;
using Project;
namespace Something {
	public class Anything {
		public static void Main(string[] args) {
			Console.WriteLine(""Script: My arguments:"");
			for (int i = 0; i < args.Length; i++){
				Console.WriteLine(@"""""""" + args[i] + @"""""""");
			}
			Console.WriteLine(""Script: Hello compiler!"");
			Tests.Test(""I am calling back my compiler."");
		}
	}
}";



		/// <summary>
		/// Callback to test
		/// </summary>
		/// <param name="msg">Message to print on console</param>
		public static void Test(string msg) {
			Console.WriteLine("Compiler: Callback argument:");
			Console.WriteLine(@"""" + msg + @"""");
		}



	}



	/// <summary>
	/// Compiler
	/// </summary>
	public static class Compiler {

		

		/// <summary>
		/// Compile
		/// </summary>
		/// <param name="source">Source code</param>
		/// <returns>Compiled assembly on success or null on error</returns>
		public static Assembly Compile(string source) {
			// Set compile parameters
			CompilerParameters cp = new CompilerParameters();
			cp.TreatWarningsAsErrors = false;
			cp.CompilerOptions = "/optimize";
			cp.ReferencedAssemblies.Add("System.dll");
			string exename = Path.GetFileName(Assembly.GetExecutingAssembly().CodeBase);
			cp.ReferencedAssemblies.Add(exename);
			// Compile
			CSharpCodeProvider compiler = new CSharpCodeProvider();
			CompilerResults exec = compiler.CompileAssemblyFromSource(cp, source);
			// Error: print errors
			if (exec.Errors.HasErrors) {
				string text = "Compile error: ";
				foreach (CompilerError ce in exec.Errors) {
					text += "\r\n" + ce.ToString();
				}
				Console.WriteLine(text);
				return null;
			}
			// Success
			return exec.CompiledAssembly;
		}



		/// <summary>
		/// Find Main
		/// </summary>
		/// <param name="a">Assembly</param>
		/// <returns>Pointer to Main on success or null on error</returns>
		public static MethodInfo FindMain(Assembly assembly) {
			if (assembly != null) {
				foreach (Module m in assembly.GetModules()) {
					foreach (Type t in m.GetTypes()) {
						foreach (MethodInfo mi in t.GetMethods()) {
							if (mi.IsStatic && mi.IsPublic && mi.Name == "Main") {
								ParameterInfo[] p = mi.GetParameters();
								if (p.Length == 1 && p[0].ParameterType == typeof(string[])) {
									return mi;
								}
							}
						}
					}
				}
			}
			return null;
		}



		/// <summary>
		/// Run Main
		/// </summary>
		/// <param name="main">Main (or anything else with void Main(string[] args) signature)</param>
		/// <param name="args">arguments of Main</param>
		public static void RunMain(MethodInfo main, string[] args) {
			if (main != null) {
				main.Invoke(null, new object[] { args });
			}
		}



		/// <summary>
		/// Compile and run
		/// </summary>
		/// <param name="source"></param>
		/// <param name="args"></param>
		public static void CompileAndRun(string source, string[] args) {
			RunMain(FindMain(Compile(source)), args);
		}



		/// <summary>
		/// Print the structure of an assembly
		/// </summary>
		/// <remarks>For debugging</remarks>
		/// <param name="assembly">Assembly</param>
		private static void ExpoloreAssembly(Assembly assembly) {
			Console.WriteLine("Modules in the assembly:");
			foreach (Module m in assembly.GetModules()) {
				Console.WriteLine("{0}", m.ScopeName);
				foreach (Type t in m.GetTypes()) {
					Console.WriteLine("\t{0}", t.FullName);
					foreach (MethodInfo mi in t.GetMethods()) {
						Console.WriteLine("\t\t{0}{1}",
							(mi.IsPublic ? "public " : mi.IsPrivate ? "private " : "") +
							(mi.IsStatic ? "static " : ""),
							mi.Name);
						foreach (ParameterInfo p in mi.GetParameters()) {
							Console.WriteLine("\t\t\t{0} {1}", p.ParameterType, p.Name);
						}
					}
				}
			}
		}



	}



	/// <summary>
	/// Main
	/// </summary>
	public static class Program {



		/// <summary>
		/// Main
		/// </summary>
		/// <param name="args">Not used</param>
		public static void Main(string[] args) {
			Console.WriteLine("Compiler: Compiling script ...");
			Console.WriteLine("//-----------------------------------------");
			Console.WriteLine(Tests.code);
			Console.WriteLine("//-----------------------------------------");
			Console.WriteLine("Compiler: Calling script with arguments: \"Hello script! Do something!\"");
			Console.WriteLine("//-----------------------------------------");
			Compiler.CompileAndRun(Tests.code, new[] { "Hello script! Do something!" });
			Console.ReadKey();
		}



	}



}